function hellip = hyperellipsoid(varargin)
%HYPERELLIPSOID Hyperellipsoid class constructor. 
%
% Syntax:      hellip = hyperellipsoid(mu, sig, c)
%
% Inputs:
%              mu is the vector for the center point
%              sig is the matrix determining the shape; for elliptical
%                probability distributions, sig will be the
%                inverse dispersion matrix 
%              c is the scalar determining the size
%
% Outputs:
%              hellip - a new hyperellipsoid object
%
% Example: 
%              % Create a new ellipsoid:
%              hellip = hyperellipsoid(mu, sig, c);
%              % Find the distance to the special point:
%              dist = distance_to_ellipsoid(hellip, point);
%              disp(['The distance is: ' num2str(dist)]);
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    switch nargin
    case 0
       %% if no input arguments, create a 3-D sphere
       hellip.center = zeros(3, 1);
       hellip.shape = eye(3);
       hellip.size = 1;
       hellip = class(hellip,'hyperellipsoid');
    case 1
       %% if single argument of class hyperellipsoid, return it
       if isa(varargin{1},'hyperellipsoid')
          hellip = varargin{1};
       else
          error('Input argument is not a hyperellipsoid object');
       end
    case 3
       %% create object using specified values
       hellip.center = varargin{1};
       hellip.shape = varargin{2};
       hellip.size = varargin{3};
       if (hellip.size<0)
          error('Negative size parameter not allowed');
       end
       eigen = eig(inv(hellip.shape));
       d = length(eigen);
       for i=1:d;
           % See Strang, p. 250:
           if (eigen(i)<=0)
              error('Shape matrix is not positive definite');
           end
       end
       [Rcent, Ccent] = size(varargin{1});
       d = Rcent;
       if (Ccent~=1)
          hellip.center
          error('Wrong number of colums in center vector (see above).');
       end
       [Rshap, Cshap] = size(varargin{2});
       if (Rshap~=d)
          hellip.center
          hellip.shape
          error('Wrong number of rows in shape matrix (see above).');
       end
       if (Cshap~=d)
          hellip.center
          hellip.shape
          error('Wrong number of columns in shape matrix (see above).');
       end
       [Rsize, Csize] = size(varargin{3});
       if (Rsize~=1)
          hellip.center
          hellip.size
          error('Wrong number of rows in size scalar (see above).');
       end
       if (Csize~=1)
          hellip.center
          hellip.size
          error('Wrong number of columns in size scalar (see above).');
       end

       hellip = class(hellip,'hyperellipsoid');
    otherwise
       %% Cannot cope with this number of inputs 
       error('Wrong number of input arguments');
    end;
end

