function [V] = vertices(hellipse)
%VERTICES Finds the d d-dimensional positive vertices for the 
%                 given ellipsoid.  A "positive" vertex is one where a 
%                 principal axis for the ellipsoid intersects the surface 
%                 of the ellipsoid in the direction of the corresponding
%                 eigenvector.  (Recall that each of the eigenvectors of 
%                 the ellipsoid's shape matrix is collinear with one of the 
%                 principal axes.)  This function does not assume that the
%                 ellipsoid is centered at the origin.  Because the   
%                 direction of each unit eigenvector is arbitrary (i.e.,
%                 multiplication by -1 still yields a unit eigenvector), a
%                 simple algorithm is used to pick a consistent
%                 orientation for the vertex points.
%
% Syntax:      [V] = vertices(hellipse)
%
% Inputs:
%              hellipse - defines the polar vertices
%
% Outputs:
%              V - d x d array with each column a positive vertex
%
% Example: 
%              A = [ 0.56569, 0.00000, 0.00000; 
%                    0.56569, 0.21213, 0.00000; 
%                   -0.56569, 0.21213, 0.44567;];
%              mu = zeros(3, 1);
%              e = hyperellipsoid(mu, A'*A, 1);
%              [S, lambda] = eig(sig);
%              V = vertices(e);
%              for i=1:length(mu);
%                 Vi = S(:,i);
%                 Vj = V(:,i);
%                 cosine = Vi'*Vj/(norm(Vi)*norm(Vj));
%                 assertTrue('Collin. w/eigs', abs(1-abs(cosine))<EPSILON);
%              end
%
% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
% EITHER EXPRESS, IMPLIED, STATUTORY OR OTHERWISE INCLUDING BUT NOT LIMITED TO
% THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, FREE OF
% DEFECTS AND NONINFRINGEMENT. 
% 
% UNDER NO CIRCUMSTANCES AND UNDER NO LEGAL THEORY OF LIABILITY, WHETHER TORT
% (INCLUDING NEGLIGENCE), CONTRACT OR OTHERWISE SHALL THE AUTHORS OR THEIR 
% EMPLOYERS BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR 
% CONSEQUENTIAL DAMAGES OF ANY KIND, INCLUDING, WITHOUT LIMITATION, DAMAGES 
% FOR LOST PROFITS, LOSS OF GOODWILL, WORK STOPPPAGE, COMPUTER FAILURE OR 
% MALFUNCTION, LOSS OF USE, LOSS OF DATA, OR ANY AND ALL OTHER COMMERCIAL 
% DAMAGES AND LOSSES WHETHER OR NOT ADVISED OF THE POSSIBILITY OF SUCH DAMAGES, 
% ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. 
% THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH 
% YOU. IF THE SOFTWARE PROVES DEFECTIVE IN ANY RESPECT, YOU ASSUME THE COST OF 
% ANY NECESSARY SERVICING, REPAIR OR CORRECTION  
% THIS SOFTWARE IS RELEASED INTO THE PUBLIC DOMAIN, OCT 2012.

% Author: Mark D. Flood
% Research and Analysis Center, Office of Financial Research
% Oct 2012; Last revision: 21-Nov-2010

%------------- BEGIN CODE --------------
    %% First, we de-mean the ellipsoid, to simplify things
    [ellipsoid2, mu] = center_at_origin(hellipse);
    sig = get(ellipsoid2, 'shape');
    size = get(ellipsoid2, 'size');
    [S, lambda] = eig(sig);
    d = length(mu);
    
    %% Algorithmically choose a specific orientation for the eigenvectors, 
    % i.e., for the columns of S.  For each column of S (i.e., for each
    % eigenvector), invert the vector if the sum of its components is 
    % negative.  This ensures some consistency in the vertices chosen.
    orient = eye(d,d);
    for j=1:d;
        if (S(:,j)'*ones(d,1)<0)
            orient(j,j) = -1;
        end
    end;
    S = S*orient;
    
    %% Calculating vertex lengths
    L = zeros(d);
    for i=1:d;
        L(i,i) = sqrt(size/lambda(i,i));
    end;
    V = S*L;
    
    %% Here, we add back in the mean:
    for i=1:d;
        V(:,i) = V(:,i) + mu(:,1);
    end;
end
